//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
// ClrHost.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

void ReportError(HRESULT hr)
{
	LPVOID lpMsgBuf;
	DWORD bOk = FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM | 
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		hr,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL); 

	if (bOk)
	{
		_ftprintf(stderr, _T("\t%s\n"), (LPTSTR)((LPTSTR)lpMsgBuf));
		LocalFree( lpMsgBuf );
	}
	else
	{
		_ftprintf(stderr, _T("Unknown Error\n"));
	}

}

void Start()
{
	printf("Unmanaged hosting code started...\n");
	//
	// Declare variables passed to CorBindToRuntime.  This sample is hard coded
	// to load the released version of the CLR, workstation build.
	//
	LPWSTR pszVer = L"v1.0.3705";
	LPWSTR pszFlavor = L"wks";
	ICorRuntimeHost *pHost = NULL;

	//
	// CorBindToRuntime is the primary api hosts use to load the CLR into a process.
	// In addition to version and "svr vs wks" - concurrent gc, and "no
	// domain neutral code ("single domain host") is selected.
	// 
	HRESULT hr = CorBindToRuntimeEx(
                      //version
                      pszVer,       
                      // svr or wks                        
                      pszFlavor,    
                      //domain-neutral"ness" and gc settings - see below.
                      STARTUP_LOADER_OPTIMIZATION_SINGLE_DOMAIN | STARTUP_CONCURRENT_GC, 
                      CLSID_CorRuntimeHost, 
                      IID_ICorRuntimeHost, 
                      (void **)&pHost);
	if (!SUCCEEDED(hr))
	{
		printf("CorBindToRuntime failed\n");
		ReportError(hr);
		return;
	}

	wchar_t buffer[128];
	DWORD dwBytes;
	hr = GetCORVersion(buffer, sizeof(buffer), &dwBytes);


	wprintf(L"Version %ls of the CLR loaded...\n", buffer);

	ICorConfiguration *pConfig = NULL;
	hr = pHost->GetConfiguration(&pConfig);
	// Call the configuration methods. . .
	// How do I get an instance of IGCHostControl?
	// IGCHostControl *pGCHostControl = NULL;
	// pConfig->SetGCHostControl();
	pConfig->Release();

	// pHost->CreateLogicalThreadState() ??
	// pHost->CreateEvidence() create evidence or IIdentity?
	// pHost->CurrentDomain(IUnknown *pAppDomain) ??
	// pHost->DeleteLogicalThreadState() ??
	// pHost->LocksHeldByLogicalThread(DWORD *pCount) ??
	// pHost->MapFile(HANDLE hFile, HMODULE *hMapAddress) ??
	// pHost->SwitchInLogicalThreadState(DWORD **pFiberCookie) ??
	// pHost->SwitchOutLogicalThreadState(DWORD **pFiberCookie) ??

	//
	// Start the CLR.
	//
	pHost->Start();
	//
	// Get a pointer to the default domain in the process.  
	//
	_AppDomain *pDefaultDomain = NULL;
	IUnknown   *pAppDomainPunk = NULL;

	hr = pHost->GetDefaultDomain(&pAppDomainPunk);
	assert(pAppDomainPunk); 

	hr = pAppDomainPunk->QueryInterface(__uuidof(_AppDomain), 
                                        (void**) &pDefaultDomain);
	assert(pDefaultDomain);

	GetCurrentDirectory(sizeof(buffer), buffer);
	wprintf(L"Current directory \"%ls\"\n", buffer);

	BSTR bstrVal;
	pDefaultDomain->get_BaseDirectory(&bstrVal);
	wprintf(L"Base directory \"%ls\"\n", (LPWSTR)bstr_t(bstrVal,false));
	pDefaultDomain->get_DynamicDirectory(&bstrVal);
	wprintf(L"Dynamic directory \"%ls\"\n", (LPWSTR)bstr_t(bstrVal,false));
	pDefaultDomain->get_FriendlyName(&bstrVal);
	wprintf(L"Friendly name \"%ls\"\n", (LPWSTR)bstr_t(bstrVal,false));
	pDefaultDomain->get_RelativeSearchPath(&bstrVal);
	wprintf(L"Relative search path \"%ls\"\n", (LPWSTR)bstr_t(bstrVal,false));
	//
	// Load the managed portion of our host into the default domain.
	//
	ICorRun *pMgdHost = NULL;
	_ObjectHandle *pObjHandle = NULL;
	
	hr = pDefaultDomain->CreateInstance(
			_bstr_t("MgdHost"), 
			_bstr_t("ClrHost.MgdHost.HostProcessRequest"),
			&pObjHandle); 
	if(FAILED(hr))
		return;
	printf("Managed hosting code successfully created...\n");

	assert(pObjHandle);

	VARIANT v;
	VariantInit(&v);
	hr = pObjHandle->Unwrap(&v);

	assert(v.pdispVal);
	hr = v.pdispVal->QueryInterface(__uuidof(ICorRun), 
                                    (void**) &pMgdHost);
	if(FAILED(hr))
		return;
	assert(pMgdHost);

	pMgdHost->Run(_bstr_t("DiningPhilosophersDomain"), _bstr_t("DiningPhilosophers.exe"));

	//
	// Release stuff
	//
	pAppDomainPunk->Release();
	pDefaultDomain->Release();
	pMgdHost->Release();

	//
	// Stop the CLR.  Once the CLR has been unloaded from a process, you
	// cannot load it again.
	//
	pHost->Stop();
}

int main(int argc, char* argv[])
{
	try
	{
		CoInitialize(NULL);
		Start();
	}
	catch(...)
	{
	}
	CoUninitialize();
}

